@@ -98,6 +98,9 @@ public class TaxServer {
 		long defaultSketchReads=200000;
 		boolean https=false;
 		
+		int serverNum_=0;
+		int serverCount_=1;
+		
 		//Create a parser object
 		Parser parser=new Parser();
 		
@@ -120,6 +123,17 @@ public class TaxServer {
 				https=Parse.parseBoolean(b);
 			}else if(a.equals("http")){
 				https=!Parse.parseBoolean(b);
+			}else if(a.equals("servers") || a.equals("numservers") || a.equals("servercount")){
+				serverCount_=Integer.parseInt(b);
+				assert(serverCount_>0) : arg;
+			}else if(a.equals("servernum")){
+				serverNum_=Integer.parseInt(b);
+				assert(serverNum_>=0) : arg;
+			}else if(a.startsWith("slave") && Tools.isDigit(a.charAt(a.length()-1))){
+				int num=Integer.parseInt(a.substring(5));
+				if(slaveAddress==null){slaveAddress=new ArrayList<String>(serverCount_);}
+				while(slaveAddress.size()<=num){slaveAddress.add(null);}
+				slaveAddress.set(num, b);
 			}else if(a.equals("table") || a.equals("gi") || a.equals("gitable")){
 				giTableFile=b;
 			}else if(a.equals("tree") || a.equals("taxtree")){
@@ -203,6 +217,18 @@ public class TaxServer {
 		if("auto".equalsIgnoreCase(patternFile)){patternFile=TaxTree.defaultPatternFile();}
 		if("auto".equalsIgnoreCase(sizeFile)){sizeFile=TaxTree.defaultSizeFile();}
 		
+		serverNum=AccessionToTaxid.serverNum=serverNum_;
+		serverCount=AccessionToTaxid.serverCount=serverCount_;
+		distributed=AccessionToTaxid.distributed=serverCount>1;
+		assert(serverNum<serverCount && serverNum>=0);
+		if(distributed && serverNum==0){
+			assert(slaveAddress!=null);
+			assert(slaveAddress.size()==serverCount);
+			for(int i=1; i<slaveAddress.size(); i++){
+				assert(slaveAddress.get(i)!=null);
+			}
+		}
+		
 		maxConcurrentSketchCompareThreads=Tools.mid(1, maxConcurrentSketchCompareThreads, Shared.threads());
 		maxConcurrentSketchLoadThreads=Tools.mid(1, maxConcurrentSketchLoadThreads, Shared.threads());
 		assert(maxConcurrentSketchCompareThreads>=1);
@@ -1241,7 +1267,7 @@ public class TaxServer {
 		}else if(type2==ACCESSION){
 			for(String name : names){
 				sb.append(comma);
-				int ncbi=AccessionToTaxid.get(name);
+				int ncbi=accessionToTaxid(name);
 				sb.append(ncbi);
 				comma=",";
 			}
@@ -1277,7 +1303,7 @@ public class TaxServer {
 		}else if(type2==TAXID){
 			tn=getTaxNodeTaxid(Integer.parseInt(name));
 		}else if(type2==ACCESSION){
-			int ncbi=AccessionToTaxid.get(name);
+			int ncbi=accessionToTaxid(name);
 			tn=(ncbi<0 ? null : tree.getNode(ncbi));
 		}else if(type2==HEADER || type2==SILVAHEADER){
 			tn=getTaxNodeHeader(name, type2==SILVAHEADER);
@@ -1347,7 +1373,7 @@ public class TaxServer {
 		}else if(type2==ACCESSION){
 			for(String name : names){
 				sb.append(comma);
-				final int tid=AccessionToTaxid.get(name);
+				final int tid=accessionToTaxid(name);
 				TaxNode tn=tree.getNode(tid, true);
 				if(tn==null){sb.append("Not found");}
 				else{sb.append(tree.toSemicolon(tn, skipNonCanonical, mononomial));}
@@ -1394,7 +1420,7 @@ public class TaxServer {
 		}else if(type==TAXID){
 			tn0=getTaxNodeTaxid(Integer.parseInt(name));
 		}else if(type==ACCESSION){
-			int ncbi=AccessionToTaxid.get(name);
+			int ncbi=accessionToTaxid(name);
 			tn0=(ncbi>=0 ? tree.getNode(ncbi) : null);
 		}else if(type==HEADER || type==SILVAHEADER){
 			tn0=getTaxNodeHeader(name, type==SILVAHEADER);
@@ -1538,7 +1564,7 @@ public class TaxServer {
 			}
 		}else if(type2==ACCESSION){
 			for(String name : names){
-				int ncbi=AccessionToTaxid.get(name);
+				int ncbi=accessionToTaxid(name);
 				if(ncbi>=0){list.add(ncbi);}
 				else{notFound.incrementAndGet();}
 			}
@@ -1554,6 +1580,30 @@ public class TaxServer {
 		return list;
 	}
 	
+	public static final String stripAccession(String s){
+		if(s==null){return null;}
+		s=s.toUpperCase();
+		for(int i=0; i<s.length(); i++){
+			char c=s.charAt(i);
+			if(c=='.' || c==':'){return s.substring(0, i);}
+		}
+		return s;
+	}
+	
+	private int accessionToTaxid(String accession){
+		if(accession==null){return -1;}
+		int tid=AccessionToTaxid.get(accession);
+		if(tid<0 && distributed && serverNum==0){
+			accession=stripAccession(accession);
+			int slaveNum=accession.hashCode()%serverCount;
+			if(slaveNum!=serverNum){
+				String path=slaveAddress.get(slaveNum);
+				tid=TaxClient.accessionToTaxidSpecificServer(path, accession);
+			}
+		}
+		return tid;
+	}
+	
 	/** Look up a TaxNode by parsing the organism name */
 	TaxNode getTaxNodeByName(String name){
 		if(verbose2){System.err.println("Fetching node for "+name);}
@@ -2225,7 +2275,7 @@ public class TaxServer {
 	private String sizeFile=null;
 
 	/** Location of sequence directory tree */
-	private String basePath="/global/projectb/sandbox/gaag/bbtools/tree/";
+	private String basePath="/global/cfs/cdirs/bbtools/tree/";
 	
 	/** Used for taxonomic tree traversal */
 	private final TaxTree tree;
@@ -2255,6 +2305,11 @@ public class TaxServer {
 	/*----------------         Final Fields         ----------------*/
 	/*--------------------------------------------------------------*/
 
+	private final boolean distributed;
+	private final int serverNum;
+	private final int serverCount;
+	private ArrayList<String> slaveAddress;
+	
 	public final String favIconPath=Data.findPath("?favicon.ico");
 	public final byte[] favIcon=ReadWrite.readRaw(favIconPath);
 	
